<?php
/**
 * Wicked Abtract Page Class.
 *
 * $Horde: wicked/lib/Page.php,v 1.59 2004/12/01 16:16:50 eraserhd Exp $
 *
 * @author  Tyler Colbert <tyler-hordeml@colberts.us>
 * @version $Revision: 1.59 $
 * @package Wicked
 */
class Page {

    /**
     * Display modes supported by this page. Possible modes:
     *
     *   WICKED_MODE_CONTENT
     *   WICKED_MODE_DISPLAY
     *   WICKED_MODE_EDIT
     *   WICKED_MODE_REMOVE
     *   WICKED_MODE_HISTORY
     *   WICKED_MODE_DIFF
     *   WICKED_MODE_INFO
     *   WICKED_MODE_DEBUG
     *   WICKED_MODE_LOCKING
     *   WICKED_MODE_CREATE
     *
     * @var array $supportedModes
     */
    var $supportedModes = array();

    /**
     * Instance of a Text_Wiki processor.
     *
     * @var object Text_Wiki $_proc
     */
    var $_proc;

    /**
     * The loaded page info.
     *
     * @var array $_page
     */
    var $_page;

    /**
     * Is this a validly loaded page?
     *
     * @return boolean  True if we've loaded data, false otherwise.
     */
    function isValid()
    {
        return !empty($this->_page) && !is_a($this->_page, 'PEAR_Error');
    }

    /**
     * Retrieve this user's permissions for this page. If the object
     * does not exist, we assume reasonable defaults.
     *
     * @return integer  The permissions bitmask.
     */
    function getPermissions()
    {
        global $perms;

        $permName = 'wicked:pages:' . $this->pageName();
        if ($perms->exists($permName)) {
            return $perms->getPermissions($permName);
        } elseif ($perms->exists('wicked:pages')) {
            return $perms->getPermissions('wicked:pages');
        } else {
            if (!Auth::getAuth()) {
                return PERMS_SHOW | PERMS_READ;
            } else {
                return PERMS_SHOW | PERMS_READ | PERMS_EDIT | PERMS_DELETE;
            }
        }
    }

    /**
     * Returns if the page allows a mode. Access rights and user state
     * are taken into consideration.
     *
     * @see $supportedModes
     *
     * @param integer $mode  The mode to check for.
     *
     * @return boolean  True if the mode is allowed.
     */
    function allows($mode)
    {
        global $browser;

        $pagePerms = $this->getPermissions();

        switch ($mode) {
        case WICKED_MODE_CREATE:
            // Special mode for pages that don't exist yet - generic
            // to all pages.
            if ($browser->isRobot()) {
                return false;
            }

            if (Auth::isAdmin()) {
                return true;
            }

            global $perms;
            $permName = 'wicked';
            if ($perms->exists($permName)) {
                return $perms->getPermissions($permName) & PERMS_EDIT;
            } else {
                return Auth::getAuth();
            }
            break;

        case WICKED_MODE_EDIT:
            if (Auth::isAdmin()) {
                return true;
            }

            if (($pagePerms & PERMS_EDIT) == 0) {
                return false;
            }

            /* Locked page. */
            if ($this->isLocked()) {
                return false;
            }
            break;

        case WICKED_MODE_REMOVE:
            if (Auth::isAdmin()) {
                return true;
            }

            if (($pagePerms & PERMS_DELETE) == 0) {
                return false;
            }
            break;

        case WICKED_MODE_LOCKING:
            if ($browser->isRobot()) {
                return false;
            }

            if (Auth::isAdmin()) {
                return true;
            }

            if (($pagePerms & PERMS_EDIT) == 0) {
                return false;
            }
            break;

        case WICKED_MODE_DEBUG:
            if (!Auth::isAdmin()) {
                return false;
            }
            break;

        case WICKED_MODE_CONTENT:
        case WICKED_MODE_DISPLAY:
        case WICKED_MODE_BLOCK:
            if (Auth::isAdmin()) {
                return true;
            }

            if (($pagePerms & PERMS_READ) == 0) {
                return false;
            }
            break;
        }

        return $this->supports($mode);
    }

    /**
     * See if the page supports a particular mode.
     * @see $supportedModes
     *
     * @param integer $mode      Which mode to check for
     *
     * @return boolean            True or false
     */
    function supports($mode)
    {
        return !empty($this->supportedModes[$mode]);
    }

    /**
     * Get the page we are currently on.
     *
     * @return  Returns a Page or PEAR_Error.
     */
    function &getCurrentPage()
    {
        return Page::getPage(Util::getFormData('page'),
                             Util::getFormData('version'),
                             Util::getFormData('autocreate'),
                             Util::getFormData('referrer'),
                             Util::getFormData('template'));
    }

    /**
     * Get the page we are currently on.
     *
     * @return mixed  Returns a Page or PEAR_Error.
     */
    function &getPage($pagename, $pagever = null, $autocreate = false, $referrer = null, $template = null)
    {
        global $conf;

        if (empty($pagename)) {
            $pagename = 'WikiHome';
        }

        $file = WICKED_BASE . '/lib/Page/' . $pagename . '.php';
        if (file_exists($file)) {
            require_once $file;
            return $page = &new $pagename($referrer);
        }

        require_once WICKED_BASE . '/lib/Page/StandardPage.php';

        /* If we have a version, but it is actually the most recent version,
         * ignore it. */
        if (!empty($pagever)) {
            $page = &new StandardPage($pagename, false, null);
            if ($page->isValid() && $page->version() == $pagever) {
                return $page;
            }
            require_once WICKED_BASE . '/lib/Page/StandardPage/StdHistoryPage.php';
            return $page = &new StdHistoryPage($pagename, $pagever, $autocreate);
        }

        $page = &new StandardPage($pagename, $autocreate, $template);
        if (!$page->isValid() && $page->allows(WICKED_MODE_EDIT)) {
            require_once WICKED_BASE . '/lib/Page/AddPage.php';
            $page = &new AddPage($pagename);
        }
        return $page;
    }

    function versionCreated()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function formatVersionCreated()
    {
        global $prefs;
        $v = $this->versionCreated();
        if (is_a($v, 'PEAR_Error') || !$v) {
            return _("Never");
        } else {
            return strftime($prefs->getValue('date_format'), $v);
        }
    }

    function author()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function hits()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function version()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Retrieve the previous version number for this page
     *
     * @return mixed A string containing the previous version or null if this
     *               is the first version.
     */
    function previousVersion()
    {
        global $wicked;

        $res = $this->version();
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        $history = $wicked->getHistory($this->pageName());
        if (is_a($history, 'PEAR_Error')) {
            return $history;
        }

        if (count($history) == 0) {
            return null;
        }
        if ($this->isOld()) {
            for ($i = 0; $i < count($history); $i++) {
                $checkver = sprintf('%d.%d', $history[$i]['page_majorversion'],
                                    $history[$i]['page_minorversion']);
                if ($checkver == $this->version()) {
                    if ($i + 1 < count($history)) {
                        $i++;
                        break;
                    } else {
                        return null;
                    }
                }
            }

            if ($i == count($history)) {
                return null;
            }
        } else {
            $i = 0;
        }

        return sprintf('%d.%d', $history[$i]['page_majorversion'],
                       $history[$i]['page_minorversion']);
    }

    function isOld()
    {
        return false;
    }

    /**
     * Render this page in Display mode. You really must override this
     * function if your page is to be anything like a real page.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function display()
    {
        $inner = $this->displayContents(false);
        if (is_a($inner, 'PEAR_Error')) {
            return $inner;
        }
        require WICKED_TEMPLATES . '/display/title.inc';
        echo $inner;
    }

    /**
     * Render this page for displaying in a block. You really must override
     * this function if your page is to be anything like a real page.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function block()
    {
        return $this->displayContents(true);
    }

    function displayContents($isBlock)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Render this page in Remove mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function remove()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Render this page in History mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function history()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Render this page in Diff mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function diff()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Render this page in Info mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function info()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Render this page in Debug mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     *
     */
    function debug()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function &getProcessor()
    {
        if (!isset($this->_proc)) {
            global $wicked;

            /* This list is based on the default from Text/Wiki.php */
            $rules = array('Prefilter',
                           'Delimiter',
                           'Code',
                           'Html',
                           'Raw',
                           /* 'Include', */
                           /* 'Embed', */
                           'Anchor',
                           'Heading',
                           'Toc',
                           'Attribute',
                           'Horiz',
                           'Break',
                           'Blockquote',
                           'List',
                           'Deflist',
                           'Table',
                           'Image',
                           'Phplookup',
                           'Center',
                           'Newline',
                           'Paragraph',
                           'Url',
                           'Freelink',
                           'Interwiki',
                           'Wikilink',
                           'Colortext',
                           'Strong',
                           'Bold',
                           'Emphasis',
                           'Italic',
                           'Tt',
                           'Superscript',
                           'Revise',
                           'Tighten',
                           'Translatehtml'
                           );

            $pages = &$wicked->getPages();
            $view_url = Util::addParameter(Horde::applicationUrl('display.php'),
                                           'referrer', $this->pageName());

            require_once 'Text/Wiki.php';
            $this->_proc = &new Text_Wiki($rules);

            $this->_proc->setRenderConf('Xhtml', 'Wikilink',
                                        array('pages' => $pages,
                                              'view_url' => Util::addParameter($view_url, 'page', ''),
                                              'new_url' => Util::addParameter($view_url, array('autocreate' => 1,
                                                                                               'page' => '')),
                                              'new_text' => Horde::img('question.png')));

            $this->_proc->setRenderConf('Xhtml', 'Freelink',
                                        array('pages' => $pages,
                                              'view_url' => Util::addParameter($view_url, 'page', ''),
                                              'new_url' => Util::addParameter($view_url, array('autocreate', 1,
                                                                                               'page' => '')),
                                              'new_text' => Horde::img('question.png')));

            $this->_proc->setRenderConf('Xhtml', 'Table',
                                        array('css_table' => 'table',
                                              'css_td' => 'table-cell',
                                              'css_th' => 'table-cell'));

            $this->_proc->addPath('parse', WICKED_BASE . '/lib/Text_Wiki/Parse');
            $this->_proc->addPath('render', WICKED_BASE . '/lib/Text_Wiki/Render');
        }

        return $this->_proc;
    }

    function render($mode, $params = null)
    {
        switch ($mode) {
        case WICKED_MODE_CONTENT:
            return $this->content($params);

        case WICKED_MODE_DISPLAY:
            return $this->display($params);

        case WICKED_MODE_BLOCK:
            return $this->block($params);

        case WICKED_MODE_REMOVE:
            return $this->remove();

        case WICKED_MODE_HISTORY:
            return $this->history();

        case WICKED_MODE_DIFF:
            return $this->diff($params);

        case WICKED_MODE_INFO:
            return $this->info();

        case WICKED_MODE_DEBUG:
            return $this->debug();

        default:
            return PEAR::raiseError(_("Unsupported"));
        }
    }

    function isLocked()
    {
        return false;
    }

    function lock()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function unlock()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function updateText($newtext, $changelog, $minorchange)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getText()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function pageName()
    {
        return null;
    }

    function referrer()
    {
        return null;
    }

    function pageUrl($linkpage = 'display.php', $actionId = null)
    {
        $url = Util::addParameter($linkpage, 'page', $this->pageName());
        if (!is_null($actionId)) {
            $url = Util::addParameter($url, 'actionID', $actionId);
        }
        return Horde::applicationUrl($url);
    }

    function removeVersion($version)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function removeAllVersions()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function handleAction()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

}
